<p>You are sharing: </p>
<p class="project-path"></p>
<p>&nbsp;</p>
<p>Search by name or username.</p>
<p>Use "a:" for secondary accounts.</p>
<p>&nbsp;</p>
<div class="auto-complete-wrapper">
    <i class="fa fa-circle-o-notch fa-spin loading"></i>
    <i class="fa fa-exclamation-circle error"></i>
    <input type="text" id="name-search" class="form-control" name="name" placeholder="Start typing to add names...">
</div>
<h2>Shared with</h2>
<ul id="shared-people"></ul>